										

function[KK,PP]=doub(FF,HH,QQ,RR)

%DOUBLE.M

%function[KK,PP]=double(FF,HH,QQ,RR)

%  This program uses the "doubling algorithm" to solve the
%  Riccati matrix difference equations associated with the
%  Kalman filter.  FF is nxn, HH is kxn, QQ is nxn, RR is kxk.
%  The program returns the gain KK and the stationary covariance
%  matrix of the one-step ahead errors in forecasting the state.
%
%  The program creates the Kalman filter for the following system:
%
%       x(t+1) = FF * x(t) + e(t+1)
%
%         y(t) = HH * x(t) + v(t)
%
%  where E e(t+1)*e(t+1)' =  QQ, and E v(t)*v(t)' = RR, and v(s) is orthogonal
%  to e(t) for all t and s.  The program creates the observer system
%
%        xx(t+1) = FF * xx(t) + KK * a(t)
%           y(t) = HH * xx(t) + a(t),
%
%  where KK is the Kalman gain ,PP = E (x(t) - xx(t))*(x(t) - xx(t))', and
%  a(t) = y(t) - E[y(t)| y(t-1), y(t-2), ... ], and xx(t)=E[x(t)|y(t-1),...].
%  NOTE:  By using DUALITY, control problems can also be solved.




a0=FF';

b0=HH'*(RR\HH);

g0=QQ;

tol=1e-15;

dd=1;

ss=max(size(FF));

v=eye(ss);

while dd>tol 

a1=a0 *((v+b0*g0)\a0);

b1=b0+a0*((v+b0*g0)\(b0*a0'));

g1=g0+a0'*g0*((v+b0*g0)\a0);

k1=FF*g1*HH'/(HH*g1*HH'+RR);

k0=FF*g0*HH'/(HH*g0*HH'+RR);

dd=max(max(abs(k1-k0)));

a0=a1;

b0=b1;

g0=g1;

end



KK=k1;

PP=g1;